import { RangeSet } from '@codemirror/state';
import { GutterMarker, EditorView, gutter } from '@codemirror/view';
import { getCurStatements } from '@tidbcloud/codemirror-extension-cur-sql';
import { getNearbyStatement } from '@tidbcloud/codemirror-extension-sql-parser';

function getMarkers(view, curSqlMarker, config) {
    let markers = RangeSet.empty;
    // when something happens, hide the gutter
    if (config.whenHide && config.whenHide(view)) {
        return markers;
    }
    let curStatements = getCurStatements(view.state);
    if (curStatements[0].content === '') {
        const nearby = getNearbyStatement(view.state, curStatements[0].from);
        if (nearby) {
            curStatements = [nearby];
        }
        else {
            return markers;
        }
    }
    try {
        const startPos = [];
        curStatements.forEach((s) => {
            for (let i = s.lineFrom; i <= s.lineTo; i++) {
                const l = view.state.doc.line(i);
                startPos.push(l.from);
            }
        });
        const startPosSet = [...new Set(startPos)];
        startPosSet.forEach((pos) => (markers = markers.update({ add: [curSqlMarker.range(pos)] })));
    }
    catch (error) {
        console.log('cur-sql-gutter', error);
    }
    return markers;
}
/**
 * gutter style
 */
const baseTheme = (config) => {
    return EditorView.baseTheme({
        '.cm-sql-gutter .cm-gutterElement': {
            width: `${config.width ?? 2}px`
        },
        '.cm-lineNumbers .cm-gutterElement': {
            paddingLeft: '8px',
            paddingRight: '8px'
        }
    });
};
const sqlGutter = (curSqlMarker, config) => {
    return gutter({
        class: `cm-sql-gutter ${config.className || ''}`,
        initialSpacer: () => curSqlMarker,
        markers: (view) => getMarkers(view, curSqlMarker, config)
    });
};
function curSqlGutter(config = {}) {
    const curSqlMarker = new (class extends GutterMarker {
        toDOM() {
            const el = document.createElement('div');
            el.style.background = config.backgroundColor || '#0CA6F2';
            el.style.height = '100%';
            return el;
        }
    })();
    return [baseTheme(config), sqlGutter(curSqlMarker, config)];
}

export { curSqlGutter };
